@@ -1,6 +1,6 @@
 /*
  *   teamd_lw_arp_ping.c - Team port arp ping link watcher
- *   Copyright (C) 2012-2013 Jiri Pirko <jiri@resnulli.us>
+ *   Copyright (C) 2012-2015 Jiri Pirko <jiri@resnulli.us>
  *   Copyright (C) 2014 Erik Hugne <erik.hugne@ericsson.com>
  *
  *   This library is free software; you can redistribute it and/or
@@ -55,6 +55,8 @@ lw_ap_ppriv_get(struct lw_psr_port_priv *psr_ppriv)
 	in_struct_offset(struct arphdr, ar_op)
 
 static struct sock_filter arp_rpl_flt[] = {
+	BPF_STMT(BPF_LD + BPF_B + BPF_ABS, SKF_AD_OFF + SKF_AD_PROTOCOL),
+	BPF_JUMP(BPF_JMP + BPF_JEQ + BPF_K, ETH_P_ARP, 0, 4),
 	BPF_STMT(BPF_LD + BPF_H + BPF_ABS, OFFSET_ARP_OP_CODE),
 	BPF_JUMP(BPF_JMP + BPF_JEQ + BPF_K, ARPOP_REPLY, 1, 0),
 	BPF_JUMP(BPF_JMP + BPF_JEQ + BPF_K, ARPOP_REQUEST, 0, 1),
@@ -68,6 +70,8 @@ static const struct sock_fprog arp_rpl_fprog = {
 };
 
 static struct sock_filter arp_novlan_rpl_flt[] = {
+	BPF_STMT(BPF_LD + BPF_B + BPF_ABS, SKF_AD_OFF + SKF_AD_PROTOCOL),
+	BPF_JUMP(BPF_JMP + BPF_JEQ + BPF_K, ETH_P_ARP, 0, 6),
 	BPF_STMT(BPF_LD + BPF_B + BPF_ABS, SKF_AD_OFF + SKF_AD_VLAN_TAG_PRESENT),
 	BPF_JUMP(BPF_JMP + BPF_JEQ + BPF_K, 0, 0, 4),
 	BPF_STMT(BPF_LD + BPF_H + BPF_ABS, OFFSET_ARP_OP_CODE),
@@ -83,6 +87,8 @@ static const struct sock_fprog arp_novlan_rpl_fprog = {
 };
 
 static struct sock_filter arp_vlan_rpl_flt[] = {
+	BPF_STMT(BPF_LD + BPF_B + BPF_ABS, SKF_AD_OFF + SKF_AD_PROTOCOL),
+	BPF_JUMP(BPF_JMP + BPF_JEQ + BPF_K, ETH_P_ARP, 0, 8),
 	BPF_STMT(BPF_LD + BPF_B + BPF_ABS, SKF_AD_OFF + SKF_AD_VLAN_TAG_PRESENT),
 	BPF_JUMP(BPF_JMP + BPF_JEQ + BPF_K, 0, 6, 0),
 	BPF_STMT(BPF_LD + BPF_B + BPF_ABS, SKF_AD_OFF + SKF_AD_VLAN_TAG),
@@ -95,7 +101,7 @@ static struct sock_filter arp_vlan_rpl_flt[] = {
 };
 
 /* this hack replaces vlanid value in filter code */
-#define SET_FILTER_VLANID(fprog, vlanid) (fprog)->filter[3].k = vlanid
+#define SET_FILTER_VLANID(fprog, vlanid) (fprog)->filter[5].k = vlanid
 
 static const struct sock_fprog arp_vlan_rpl_fprog = {
 	.len = ARRAY_SIZE(arp_vlan_rpl_flt),
@@ -118,7 +124,7 @@ static int set_in_addr(struct in_addr *addr, const char *hostname)
 static char *str_in_addr(struct in_addr *addr)
 {
 	struct sockaddr_in sin;
-	char buf[NI_MAXHOST];
+	static char buf[NI_MAXHOST];
 
 	memcpy(&sin.sin_addr, addr, sizeof(*addr));
 	return __str_sockaddr((struct sockaddr *) &sin, sizeof(sin), AF_INET,
@@ -142,7 +148,7 @@ static int lw_ap_sock_open(struct lw_psr_port_priv *psr_ppriv)
 	}
 	return teamd_packet_sock_open(&psr_ppriv->sock,
 				      psr_ppriv->common.tdport->ifindex,
-				      htons(ETH_P_ARP), &fprog, &arp_rpl_fprog);
+				      htons(ETH_P_ALL), &fprog, &arp_rpl_fprog);
 }
 
 static void lw_ap_sock_close(struct lw_psr_port_priv *psr_ppriv)
@@ -292,6 +298,7 @@ static int lw_ap_send(struct lw_psr_port_priv *psr_ppriv)
 				    0, (struct sockaddr *) &ll_bcast,
 				    sizeof(ll_bcast));
 	} else {
+		ll_bcast.sll_protocol = htons(ETH_P_ARP);
 		return teamd_sendto(psr_ppriv->sock, &ap, sizeof(ap),
 				    0, (struct sockaddr *) &ll_bcast,
 				    sizeof(ll_bcast));
