@@ -870,20 +870,25 @@ static int lacp_port_partner_update(struct lacp_port *lacp_port)
 	return 0;
 }
 
+static void lacp_port_actor_system_update(struct lacp_port *lacp_port)
+{
+	struct lacpdu_info *actor = &lacp_port->actor;
+
+	memcpy(actor->system, lacp_port->ctx->hwaddr, ETH_ALEN);
+}
+
 static void lacp_port_actor_init(struct lacp_port *lacp_port)
 {
 	struct lacpdu_info *actor = &lacp_port->actor;
 
 	actor->system_priority = htons(lacp_port->lacp->cfg.sys_prio);
-	memcpy(actor->system, lacp_port->ctx->hwaddr, ETH_ALEN);
-        actor->key = htons(lacp_port->cfg.lacp_key);
-        actor->port_priority = htons(lacp_port->cfg.lacp_prio);
+	actor->key = htons(lacp_port->cfg.lacp_key);
+	actor->port_priority = htons(lacp_port->cfg.lacp_prio);
 	actor->port = htons(lacp_port->tdport->ifindex);
+	lacp_port_actor_system_update(lacp_port);
 }
 
-static int lacpdu_send(struct lacp_port *lacp_port);
-
-static int lacp_port_actor_update(struct lacp_port *lacp_port)
+static void lacp_port_actor_update(struct lacp_port *lacp_port)
 {
 	uint8_t state = 0;
 
@@ -903,9 +908,10 @@ static int lacp_port_actor_update(struct lacp_port *lacp_port)
 	teamd_log_dbg("%s: lacp info state: 0x%02X.", lacp_port->tdport->ifname,
 						      state);
 	lacp_port->actor.state = state;
-	return lacpdu_send(lacp_port);
 }
 
+static int lacpdu_send(struct lacp_port *lacp_port);
+
 static int lacp_port_set_state(struct lacp_port *lacp_port,
 			       enum lacp_port_state new_state)
 {
@@ -960,7 +966,10 @@ static int lacp_port_set_state(struct lacp_port *lacp_port,
 	if (err)
 		return err;
 
-	return lacp_port_actor_update(lacp_port);
+	lacp_port_actor_update(lacp_port);
+	if (lacp_port->periodic_on)
+		return 0;
+	return lacpdu_send(lacp_port);
 }
 
 static enum lacp_port_state lacp_port_get_state(struct lacp_port *lacp_port)
@@ -1005,6 +1014,11 @@ static int lacpdu_send(struct lacp_port *lacp_port)
 	struct sockaddr_ll ll_my;
 	struct sockaddr_ll ll_slow;
 	int err;
+	bool admin_state;
+
+	admin_state = team_get_ifinfo_admin_state(lacp_port->ctx->ifinfo);
+	if (!admin_state)
+		return 0;
 
 	err = teamd_getsockname_hwaddr(lacp_port->sock, &ll_my, 0);
 	if (err)
@@ -1056,7 +1070,8 @@ static int lacpdu_recv(struct lacp_port *lacp_port)
 		return err;
 
 	/* Check if the other side has correct info about us */
-	if (memcmp(&lacpdu.partner, &lacp_port->actor,
+	if (!lacp_port->periodic_on &&
+	    memcmp(&lacpdu.partner, &lacp_port->actor,
 		   sizeof(struct lacpdu_info))) {
 		err = lacpdu_send(lacp_port);
 		if (err)
@@ -1097,6 +1112,7 @@ static int lacp_callback_periodic(struct teamd_context *ctx, int events,
 {
 	struct lacp_port *lacp_port = priv;
 
+	lacp_port_actor_update(lacp_port);
 	return lacpdu_send(lacp_port);
 }
 
@@ -1275,21 +1291,43 @@ static const struct teamd_port_priv lacp_port_priv = {
 static int lacp_event_watch_hwaddr_changed(struct teamd_context *ctx,
 					   void *priv)
 {
+	struct lacp *lacp = priv;
 	struct teamd_port *tdport;
 	int err;
 
 	teamd_for_each_tdport(tdport, ctx) {
-		err = team_hwaddr_set(ctx->th, tdport->ifindex, ctx->hwaddr,
-				      ctx->hwaddr_len);
-		if (err) {
-			teamd_log_err("%s: Failed to set port hardware address.",
-				      tdport->ifname);
+		struct lacp_port *lacp_port = lacp_port_get(lacp, tdport);
+
+		err = lacp_port_set_mac(ctx, tdport);
+		if (err)
 			return err;
-		}
+		lacp_port_actor_system_update(lacp_port);
 	}
 	return 0;
 }
 
+static int lacp_event_watch_admin_state_changed(struct teamd_context *ctx,
+					        void *priv)
+{
+	struct lacp *lacp = priv;
+	struct teamd_port *tdport;
+	bool admin_state;
+	int err;
+
+	admin_state = team_get_ifinfo_admin_state(ctx->ifinfo);
+
+	teamd_for_each_tdport(tdport, ctx) {
+		struct lacp_port *lacp_port = lacp_port_get(lacp, tdport);
+
+		err = lacp_port_set_state(lacp_port,
+					  admin_state?PORT_STATE_EXPIRED:PORT_STATE_DISABLED);
+		if (err)
+			return err;
+	}
+	return 0;
+}
+
+
 static int lacp_event_watch_port_added(struct teamd_context *ctx,
 				       struct teamd_port *tdport, void *priv)
 {
@@ -1319,11 +1357,12 @@ static int lacp_event_watch_port_changed(struct teamd_context *ctx,
 	return lacp_port_link_update(lacp_port);
 }
 
-static const struct teamd_event_watch_ops lacp_port_watch_ops = {
+static const struct teamd_event_watch_ops lacp_event_watch_ops = {
 	.hwaddr_changed = lacp_event_watch_hwaddr_changed,
 	.port_added = lacp_event_watch_port_added,
 	.port_removed = lacp_event_watch_port_removed,
 	.port_changed = lacp_event_watch_port_changed,
+	.admin_state_changed = lacp_event_watch_admin_state_changed,
 };
 
 static int lacp_carrier_init(struct teamd_context *ctx, struct lacp *lacp)
@@ -1804,7 +1843,7 @@ static int lacp_init(struct teamd_context *ctx, void *priv)
 		teamd_log_err("Failed to initialize carrier.");
 		return err;
 	}
-	err = teamd_event_watch_register(ctx, &lacp_port_watch_ops, lacp);
+	err = teamd_event_watch_register(ctx, &lacp_event_watch_ops, lacp);
 	if (err) {
 		teamd_log_err("Failed to register event watch.");
 		return err;
@@ -1824,7 +1863,7 @@ static int lacp_init(struct teamd_context *ctx, void *priv)
 balancer_fini:
 	teamd_balancer_fini(lacp->tb);
 event_watch_unregister:
-	teamd_event_watch_unregister(ctx, &lacp_port_watch_ops, lacp);
+	teamd_event_watch_unregister(ctx, &lacp_event_watch_ops, lacp);
 	return err;
 }
 
@@ -1834,7 +1873,7 @@ static void lacp_fini(struct teamd_context *ctx, void *priv)
 
 	teamd_state_val_unregister(ctx, &lacp_state_vg, lacp);
 	teamd_balancer_fini(lacp->tb);
-	teamd_event_watch_unregister(ctx, &lacp_port_watch_ops, lacp);
+	teamd_event_watch_unregister(ctx, &lacp_event_watch_ops, lacp);
 	lacp_carrier_fini(ctx, lacp);
 }
 
